"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Storage = require("@google-cloud/storage");
class GoogleStorageGeneric {
    constructor(config) {
        this.config = config;
    }
    storage() {
        return new Storage({
            projectId: this.config.project_id,
            keyFilename: this.config.key_filename
        });
    }
    getPublicUrl(filename) {
        if (this.config.host) {
            return `${this.config.host}/${filename}`;
        }
        else {
            return `https://storage.googleapis.com/${this.config.bucket}/${filename}`;
        }
    }
    upload(reqFile) {
        const bucket = this.storage().bucket(this.config.bucket);
        const name = `${Date.now()}_${reqFile.originalname}`;
        return new Promise((resolve, reject) => {
            bucket
                .file(name)
                .createWriteStream({
                metadata: {
                    contentType: reqFile.mimetype
                }
            })
                .on('error', (err) => {
                return resolve({
                    status: 'failure',
                    failure_message: err.toString()
                });
            })
                .on('finish', () => {
                return resolve({
                    status: 'success',
                    url: this.getPublicUrl(name)
                });
            })
                .end(reqFile.buffer);
        });
    }
    uploadFile(file) {
        const bucket = this.storage().bucket(this.config.bucket);
        return new Promise((resolve, reject) => {
            bucket.upload(file.src, (err, resFile) => {
                if (err) {
                    return resolve({
                        url: file.url,
                        status: 'failure',
                        failure_message: err.toString()
                    });
                }
                const proxySchema = {
                    status: 'success',
                    url: this.getPublicUrl(resFile.id),
                    file_details: resFile
                };
                return resolve(proxySchema);
            });
        });
    }
    uploadFiles(files) {
        return Promise.all(files.map(file => {
            return this.uploadFile(file);
        }));
    }
}
exports.GoogleStorageGeneric = GoogleStorageGeneric;
